<?php

namespace Drupal\extra_siteinfo;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\node\Entity\NodeType;

/**
 * A Drupal service with business logic for extra siteinfo.
 */
class ExtraSiteInfoHelper {
  use StringTranslationTrait;
  use MessengerTrait;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a RegistrationAccessCheck object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Creates a query to get users to which to count.
   *
   * @return \Drupal\Core\Entity\Query\QueryInterface
   *   A query interface that gets users.
   */
  protected function userQuery(): QueryInterface {
    return $this->entityTypeManager->getStorage('user')->getQuery();
  }

  /**
   * Uses the user query to get the number of users.
   */
  public function numberOfUsers(): int {
    return $this->userQuery()
      ->accessCheck(FALSE)
      ->count()
      ->execute();
  }

  /**
   * Uses the user query to get the number of users by query.
   */
  public function numberOfUsersByQuery($key, $value): int {
    return $this->userQuery()
      ->condition($key, $value)
      ->accessCheck(FALSE)
      ->count()
      ->execute();
  }

  /**
   * Query to get the number of roles.
   */
  public function numberOfRoles(): int {
    return count(user_roles());
  }

  /**
   * Query to get the number of users by roles.
   */
  public function numberOfUsersByRoles() {
    $users_by_role = [];
    $roles = user_roles();
    foreach ($roles as $role => $value) {
      $users_by_role[$role] = $this->numberOfUsersByQuery('roles', $role);
    }
    return $users_by_role;
  }

  /**
   * Creates a query to get nodes to which to count.
   *
   * @return \Drupal\Core\Entity\Query\QueryInterface
   *   A query interface that gets users.
   */
  protected function nodeQuery(): QueryInterface {
    return $this->entityTypeManager->getStorage('node')->getQuery();
  }

  /**
   * Uses the node query to get the number of nodes.
   */
  public function numberOfNodes(): int {
    return $this->nodeQuery()
      ->accessCheck(FALSE)
      ->count()
      ->execute();
  }

  /**
   * Uses the node query to get the number of nodes by query.
   */
  public function numberOfNodesByQuery($key, $value): int {
    return $this->nodeQuery()
      ->condition($key, $value)
      ->accessCheck(FALSE)
      ->count()
      ->execute();
  }

  /**
   * Query to get the number of roles.
   */
  public function numberOfTypes(): int {
    return count(NodeType::loadMultiple());
  }

  /**
   * Query to get the number of nodes by content types.
   */
  public function numberOfNodesByTypes() {
    $nodes_by_types = [];
    $types = NodeType::loadMultiple();
    foreach ($types as $type => $value) {
      $nodes_by_types[$type] = $this->numberOfNodesByQuery('type', $type);
    }
    return $nodes_by_types;
  }

}
